/*!
 * VmeGbeConfig class for VME-GbE2
 *
 * Copyright (C) 2016
 *		Bee Beans Technologies Co.,Ltd.
 *		All rights reserved.
 */
#include "vme-gbe-config.h"
#include "rbcp.h"
#include <errno.h>
#include <string.h> // strerror

VmeGbeConfig::VmeGbeConfig(const std::string &fileName)
{
	m_pRbcp = NULL;
	m_regEvent = 0x00000004;
	m_filename = fileName;
	errno = 0;
	m_fs.open(m_filename.c_str(), std::ios::in | std::ios::binary);
	if (!m_fs) {
		int num = errno;
		std::string estr("Error - ");
		estr.append(m_filename.c_str());
		estr.append(": ");
		estr.append(strerror(num));
		throw estr;
		//std::perror(m_filename.c_str());
	}
}

VmeGbeConfig::~VmeGbeConfig()
{
	m_fs.close();
	delete m_pRbcp;
}

bool VmeGbeConfig::download(const std::string &address, int port, bool bEnabled)
{
	m_fs.seekg(0, std::ios_base::beg);
	if (!good())
		return false;

	if (!m_pRbcp)
		m_pRbcp = new Rbcp(address, port);

	while (good()) {
		unsigned short header;
		read(header);
		if (header == HEAD_END) {
			printLog("\n");
			break;
		}
		else {
			unsigned int addr;
			unsigned char len;
			read(addr);
			read(len);

			std::vector<char> data;
			data.resize(len);
			read(data);

			if (header == HEAD_INFO1) {
				printLog("\tHEAD_INFO1\n");
			}
			else if (header == HEAD_USER1) {
				printLog("\tHEAD_USER1\n");
				m_eventMsk = (unsigned char)data.at(1);
				m_idMin = (unsigned char)data.at(2);
				m_idMax = (unsigned char)data.at(3);
				m_id = m_idMin;
			}
			else {
				char buf[256];
				sprintf(buf, "\t%x: %08x %3d", header, addr, len);
				printLog(buf);
				if (header == HEAD_EVENT_ENABLE) {
					m_regEvent = addr;
					if (!bEnabled) {
						printLog(" - Skipped\n");
						continue;
					}
					setRegister(addr, data.data(), len);
					printLog(" - OK\n");
				}
				else if (header == HEAD_DOWNLOAD
						 || header == HEAD_DOWNLOAD1 || header == HEAD_DOWNLOAD2) {
					setRegister(addr, data.data(), len);
					printLog(" - OK\n");
				}
				else
					printLog(" - ignored\n");
			}
		}
	}
	return true;
}

bool VmeGbeConfig::setEventEnabled(unsigned char event)
{
	int res;
	if ( (res = m_pRbcp->Write(m_regEvent, (char *)&event, sizeof(event))) <= 0) {
		std::string estr;
		m_pRbcp->StrError(res, estr);
		puts(estr.c_str());
		delete m_pRbcp;
		m_pRbcp = NULL;
		return false;
	}
	return true;
}


bool VmeGbeConfig::clearEvent()
{
	return setEventEnabled(0);
}

unsigned char VmeGbeConfig::getID()
{
	unsigned char id = m_id;
	if (m_id == m_idMax)
		m_id = m_idMin;
	else
		m_id++;
	return id;
}

void VmeGbeConfig::read(unsigned int &val)
{
	checkValid();
	m_fs.read((char *)&val, sizeof(val));
	val = ntohl(val);
}

void VmeGbeConfig::read(unsigned short &val)
{
	checkValid();
	m_fs.read((char *)&val, sizeof(val));
	val = ntohs(val);
}

void VmeGbeConfig::read(unsigned char &val)
{
	checkValid();
	m_fs.read((char *)&val, sizeof(val));
}

void VmeGbeConfig::read(std::vector<char> &data)
{
	checkValid();
	m_fs.read(data.data(), data.size());
}

void VmeGbeConfig::setRegister(unsigned int address, const char *data, unsigned char length)
{
	try {
		int res;
		if ( (res = m_pRbcp->Write(address, data, length)) <= 0) {
			std::string estr;
			m_pRbcp->StrError(res, estr);
			throw estr;
		}
	}
	catch (std::string &estr) {
		printLog(" - NG\n");
		delete m_pRbcp;
		m_pRbcp = NULL;
		throw estr;
	}
}

void VmeGbeConfig::checkValid()
{
	if (!good()) {
		//perror("file access error");
		std::string estr("File Access Error - ");
		estr.append(strerror(errno));
		estr.append("\n--- aborted ---");
		throw estr;
	}
}

void VmeGbeConfig::printLog(const std::string &strLog)
{
#if 0
	printf("%s", strLog.c_str());
#else
	m_log.append(strLog);
#endif
}
